RiftFrames = {}

function RiftFrames:CreateFrame(params)
	local masterFrame = UI.CreateFrame(params.type, params.name, params.parent)
	
	-- Generic frame parameters
	if params.alpha then
		masterFrame:SetAlpha(params.alpha)
	end
	if params.visible ~= nil then
		masterFrame:SetVisible(params.visible)
	end
	if params.fillTo ~= nil then
		masterFrame:SetAllPoints(params.fillTo)
	end
	if params.layer then
		masterFrame:SetLayer(params.layer)
	end
	if params.background then
		bg = {r = 0, g = 0, b = 0, r = 1}
		bg.r = params.background.r or params.background[1] or 0
		bg.g = params.background.g or params.background[2] or 0
		bg.b = params.background.b or params.background[3] or 0
		bg.a = params.background.a or params.background[4] or 1
		masterFrame:SetBackgroundColor(bg.r,bg.g,bg.b,bg.a)
	end
	if params.height then
		masterFrame:SetHeight(params.height)
	end
	if params.width then
		masterFrame:SetWidth(params.width)
	end
	if params.parent then
		masterFrame:SetParent(params.parent)
	end
	if params.align then
		if params.align.align then
			masterFrame:SetPoint(params.align.alignTo, params.align.target, params.align.align, params.align.x, params.align.y)
		else
			for k,v in pairs(params.align) do
				pos = {
					alignTo = v.alignTo or v[1],
					target = v.target or v[2],
					align = v.align or v[3],
					x = v.x or v[4] or 0,
					y = v.y or v[5] or 0
					}
				masterFrame:SetPoint(pos.alignTo, pos.target, pos.align, pos.x, pos.y)
			end
		end
	end
	
	-- Multiple frame parameters
	if params.text and masterFrame.SetText then
		masterFrame:SetText(params.text)
	end
	if params.enabled ~= nil and masterFrame.SetEnabled then
		masterFrame:SetEnabled(params.enabled)
	end

	-- Text frame parameters
	if params.font and masterFrame.SetFont then
		masterFrame:SetFont(params.font)
	end
	if params.color and masterFrame.SetFontColor then
		color = {r = 0, g = 0, b = 0, r = 1}
		color.r = params.background.r or params.background[1] or 0
		color.g = params.background.g or params.background[2] or 0
		color.b = params.background.b or params.background[3] or 0
		color.a = params.background.a or params.background[4] or 1
		masterFrame:SetFontColor(color.r,color.g,color.b,color.a)
	end
	if params.fontSize and masterFrame.SetFontSize then
		masterFrame:SetFontSize(params.fontSize)
	end
	if params.wordwrap and masterFrame.SetWordwrap then
		masterFrame:SetWordwrap(params.wordwrap)
	end
	
	-- Texture frame parameters
	if params.texture and masterFrame.SetTexture then
		src = params.texture.source or params.texture[1]
		nm = params.texture.name or params.texture[2]
		masterFrame:SetTexture(src, nm)
	end
	
	-- RiftCheckbox frame parameters
	if params.checked ~= nil and masterFrame.SetChecked then
		masterFrame:SetChecked(params.checked)
	end
	
	-- RiftSlider frame parameters
	if params.range and masterFrame.SetRange then
		range = {
			low = params.range.l or params.range.low or params.range.lo or params.range[1] or 0,
			high = params.range.h or params.range.high or params.range.hi or params.range[2] or 100
			}
		masterFrame:SetRange(range.low, range.high)
	end
	if params.position and masterFrame.SetPosition then
		masterFrame:SetPosition(params.position)
	end
	
	-- RiftWindow frame parameters
	if params.title and masterFrame.SetTitle then
		masterFrame:SetTitle(params.title)
	end

	-- Border around frame
	--   border: defines a border around the entire frame
	--   borderTop: defines a border across the top of the frame
	--   borderBottom: defines a border across the bottom of the frame
	--   borderLeft: defines a border across the left of the frame
	--   borderRight: defines a border across the right of the frame
	--   borderWidth: defines the width of the border
	if params.border or params.borderTop or params.borderBottom or params.borderLeft or params.borderRight then
		if type(params.borderWidth) ~= "table" then
			wid = params.borderWidth
			params.borderWidth = {wid,wid,wid,wid}
		end
		if not borderColor then borderColor = {1,1,1,1} end
		if params.border or params.borderTop then
			masterFrame.borderTop = RiftFrames:CreateFrame({
				type = "Frame", name = params.name.."_borderTop", parent = masterFrame,
				layer = masterFrame:GetLayer()+1, 
				background = {
					r = borderColor.r or borderColor[1] or 1,
					g = borderColor.g or borderColor[2] or 1,
					b = borderColor.b or borderColor[3] or 1,
					a = borderColor.a or borderColor[4] or 1
					},
				align = { 
					p1 = {
						alignTo = "TOPLEFT",
						target = masterFrame,
						align = "TOPLEFT",
						x = 0, 
						y = 0
						},
					p2 = {
						alignTo = "BOTTOMRIGHT",
						target = masterFrame,
						align = "TOPRIGHT",
						x = 0, 
						y = params.borderWidth.top or params.borderWidth[1] or 2
						}
					},
				})
		end
		if params.border or params.borderBottom then
			masterFrame.borderBottom = RiftFrames:CreateFrame({
				type = "Frame", name = params.name.."_borderBottom", parent = masterFrame,
				layer = masterFrame:GetLayer()+1, 
				background = {
					r = borderColor.r or borderColor[1] or 1,
					g = borderColor.g or borderColor[2] or 1,
					b = borderColor.b or borderColor[3] or 1,
					a = borderColor.a or borderColor[4] or 1
					},
				align = {
					{
						alignTo = "TOPLEFT",
						target = masterFrame,
						align = "BOTTOMLEFT",
						x = 0, 
						y = 0 - (params.borderWidth.bottom or params.borderWidth[3] or 2),
						},
					{
						alignTo = "BOTTOMRIGHT",
						target = masterFrame,
						align = "BOTTOMRIGHT",
						x = 0, 
						y = 0
						}
					},
				})
		end
		if params.border or params.borderLeft then
			masterFrame.borderLeft = RiftFrames:CreateFrame({
				type = "Frame", name = params.name.."_borderLeft", parent = masterFrame,
				layer = masterFrame:GetLayer()+1, 
				background = {
					r = borderColor.r or borderColor[1] or 1,
					g = borderColor.g or borderColor[2] or 1,
					b = borderColor.b or borderColor[3] or 1,
					a = borderColor.a or borderColor[4] or 1
					},
				align = {
					{
						alignTo = "TOPLEFT",
						target = masterFrame,
						align = "TOPRIGHT",
						x = 0 - (params.borderWidth.left or params.borderWidth[4] or 2), 
						y = 0
						},
					{
						alignTo = "BOTTOMRIGHT",
						target = masterFrame,
						align = "BOTTOMRIGHT",
						x = 0, 
						y = 0
						}
					},
				})
		end
		if params.border or params.borderRight then
			masterFrame.borderRight = RiftFrames:CreateFrame({
				type = "Frame", name = params.name.."_borderRight", parent = masterFrame,
				layer = masterFrame:GetLayer()+1, 
				background = {
					r = borderColor.r or borderColor[1] or 1,
					g = borderColor.g or borderColor[2] or 1,
					b = borderColor.b or borderColor[3] or 1,
					a = borderColor.a or borderColor[4] or 1
					},
				align = {
					{
						alignTo = "TOPLEFT",
						target = masterFrame,
						align = "TOPLEFT",
						x = 0, 
						y = 0
						},
					{
						alignTo = "BOTTOMRIGHT",
						target = masterFrame,
						align = "BOTTOMLEFT",
						x = params.borderWidth.right or params.borderWidth[2] or 2, 
						y = 0
						}
					},
				})
		end
	end
	
	return masterFrame
end


-- Frame
--   alpha (SetAlpha)
--   fillTo (SetAllPoints)
--   background (SetBackgroundColor)
--   height (SetHeight)
--   keyfocus (SetKeyFocus)
--   layer (SetLayer)
--   mousemasking (SetMouseMasking)
--   mouseoverunit (SetMousoverUnit)
--   parent (SetParent)
--   securemode (SetSecureMode)
--   visible (SetVisible)
--   width (SetWidth)
-- Context
-- Mask
-- Text
--   font (SetFont)
--   color (SetFontColor)
--   font-size (SetFontSize)
--   text (SetText)
--   wordwrap (SetWordwrap)
-- Texture
--   texture (SetTexture)
-- RiftButton
--   enabled (SetEnabled)
--   text (SetText)
-- RiftCheckbox
--   checked (SetChecked)
--   enabled (SetEnabled)
-- RiftSlider
--   enabled (SetEnabled)
--   position (SetPosition)
--   range (SetRange)
-- RiftTextfield
--   text (SetText)
-- RiftWindow
--   controller (SetController)
--   title (SetTitle)







