-- ------------------------------------------------------------------
-- Primary Variables
-- ------------------------------------------------------------------
local context = UI.CreateContext("Context")
local window = UI.CreateFrame("Frame", "MainWindow", context)
local isStopWatchRunning = false
local isTimerRunning = false
local TimerStartTime = nil
local TimerTotalTime = nil
local TimerTimeRemaining = nil

-- ------------------------------------------------------------------
-- Startup SavedVariable checks
-- ------------------------------------------------------------------
local function AddonsLoadedDone()
	if wife_windowSettings == nil then
		wife_windowSettings = WifeAggro.windowDefaults()
	end
	if wife_timerSettings == nil then
		wife_timerSettings = WifeAggro.timerDefaults()
	end
end

-- ------------------------------------------------------------------
-- Debug Functions
-- ------------------------------------------------------------------
local function initGlobals()
	wife_windowSettings = WifeAggro.windowDefaults()
	wife_timerSettings = WifeAggro.timerDefaults()
end


-- ------------------------------------------------------------------
-- Stopwatch Functions
-- ------------------------------------------------------------------
local function toggleStopwatch()
	if isTimerRunning then return false end
	if isStopWatchRunning then
		isStopWatchRunning = false
	else
		isStopWatchRunning = true
	end
end
local function toggleCountdown()
	if isStopWatchRunning then return false end
	if isTimerRunning then
		isTimerRunning = false
	else
		isTimerRunning = true
	end
end


-- ------------------------------------------------------------------
-- SimpleMeter Thievery
-- ------------------------------------------------------------------
function alt_CreateFrame(p)
	local frame = UI.CreateFrame(p.type, p.name, p.parent)
	if p.hide then
		frame:SetVisible(false)
	end
	if p.layer then
		frame:SetLayer(p.layer)
	end

	if p.type == "Text" then
		if p.fontSize then
			frame:SetFontSize(p.fontSize)
		else
			frame:SetFontSize(12)
		end
	end

	if p.fontR then
		frame:SetFontColor(p.fontR, p.fontG, p.fontB)
	end
	if p.a then
		frame:SetBackgroundColor(p.r, p.g, p.b, p.a)
	end

	if p.text then
		frame:SetText(p.text)
	end
	if p.wordwrap then
		frame:SetWordwrap(p.wordwrap)
	end

	if p.width then
		frame:SetWidth(p.width)
	elseif p.type == "Text" then
		frame:SetWidth(frame:GetFullWidth())
	end

	if p.height then
		frame:SetHeight(p.height)
	elseif p.type == "Text" then
		frame:SetHeight(frame:GetFullHeight())
	end

	if p.align then
		frame:SetPoint(p.align, p.alignTarget, p.alignTo, p.x, p.y)
	end

	if p.padTarget then
		frame:SetPoint("TOPLEFT", p.padTarget, "TOPLEFT", p.pad * -1, p.pad * -1)
		frame:SetPoint("BOTTOMRIGHT", p.padTarget, "BOTTOMRIGHT", p.pad, p.pad)
	end

	if p.border or p.borderTop or p.borderBottom or p.borderLeft or p.borderRight then
		if p.border or p.borderTop then
			p.borderTop = p.borderTop or p.border
			frame.borderTop = UI.CreateFrame("Frame", p.name .. "_btop", frame)
			frame.borderTop:SetLayer(frame:GetLayer() + 1)
			frame.borderTop:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
			frame.borderTop:SetPoint("TOPLEFT", frame, "TOPLEFT")
			frame.borderTop:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, p.borderTop)
		end

		if p.border or p.borderBottom then
			p.borderBottom = p.borderBottom or p.border
			frame.borderBot = UI.CreateFrame("Frame", p.name .. "_bbot", frame)
			frame.borderBot:SetLayer(frame:GetLayer() + 1)
			frame.borderBot:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
			frame.borderBot:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 0, p.borderBottom * -1)
			frame.borderBot:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")
		end

		if p.border or p.borderRight then
			p.borderRight = p.borderRight or p.border
			frame.borderRt = UI.CreateFrame("Frame", p.name .. "_brt", frame)
			frame.borderRt:SetLayer(frame:GetLayer() + 1)
			frame.borderRt:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
			frame.borderRt:SetPoint("TOPLEFT", frame, "TOPLEFT")
			frame.borderRt:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", p.borderRight, 0)
		end

		if p.border or p.borderLeft then
			p.borderLeft = p.borderLeft or p.border
			frame.borderLt = UI.CreateFrame("Frame", p.name .. "_blt", frame)
			frame.borderLt:SetLayer(frame:GetLayer() + 1)
			frame.borderLt:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
			frame.borderLt:SetPoint("TOPLEFT", frame, "TOPRIGHT", p.borderLeft * -1, 0)
			frame.borderLt:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")
		end
	end

	return frame;
end


-- ------------------------------------------------------------------
-- Window Creation
-- ------------------------------------------------------------------
local function altCreateWindow()
	window = RiftFrames:CreateFrame({
		type = "Frame", name = "MainWindow", parent = context,
		height = wife_windowSettings.height, 
		width = wife_windowSettings.width,
		align = {alignTo = "TOPLEFT", target = UIParent, align = "TOPLEFT", x = wife_windowSettings.posX, y = wife_windowSettings.posY, },
	})
	window.timerFrame = RiftFrames:CreateFrame({
		type = "Text", name = "WindowTimerFrame", parent = window,
		align = {alignTo = "TOPLEFT", target = window, align = "TOPLEFT", x = 0, y = 0, },
		height = 32, width = 110, 
		text = "00:00:00", fontSize = 24,
		background = {0,0,0,.6},
	})
	window.timerFrameBackground = RiftFrames:CreateFrame({
		type = "Frame", name = "WindowTimerFrameBackground", parent = window.timerFrame,
		align = {alignTo = "TOPLEFT", target = window.timerFrame, align = "TOPLEFT", x = 0, y = 0, },
		height = 32, width = 0, 
		background = {1,0,0,.6},
		layer = 3,
	})
	window.title = RiftFrames:CreateFrame({
		type = "Frame", name = "WindowTitle", parent = window,
		align = {alignTo = "TOPLEFT", target = window.timerFrame, align = "TOPRIGHT", x = 0, y = 0, },
		height = window.timerFrame:GetHeight(), width = window.timerFrame:GetHeight(), 
		background = {1,1,1,.1},
		layer = 4,
	})
	window.title.icon = {}
	window.title.icon.clock = RiftFrames:CreateFrame({
		type = "Texture", name = "WindowIcon", parent = window.title,
		align = {alignTo = "TOPLEFT", target = window.title, align = "TOPLEFT", x = 0, y = 0},
		height = window.title:GetHeight(), width = window.title:GetWidth(),
		texture = {source = "WifeAggro", name = "clock.png"},
	})
	window.title.icon.circle = RiftFrames:CreateFrame({
		type = "Texture", name = "WindowIcon", parent = window.title,
		align = {alignTo = "TOPLEFT", target = window.title, align = "TOPLEFT", x = 0, y = 0},
		height = window.title:GetHeight(), width = window.title:GetWidth(),
		texture = {source = "WifeAggro", name = "circle.png"},
		visible = false,
	})
	window.title.icon.down = RiftFrames:CreateFrame({
		type = "Texture", name = "WindowIcon", parent = window.title,
		align = {alignTo = "TOPLEFT", target = window.title, align = "TOPLEFT", x = 0, y = 0},
		height = window.title:GetHeight(), width = window.title:GetWidth(),
		texture = {source = "WifeAggro", name = "down.png"},
		visible = false,
	})
	window.title.icon.up = RiftFrames:CreateFrame({
		type = "Texture", name = "WindowIcon", parent = window.title,
		align = {alignTo = "TOPLEFT", target = window.title, align = "TOPLEFT", x = 0, y = 0},
		height = window.title:GetHeight(), width = window.title:GetWidth(),
		texture = {source = "WifeAggro", name = "up.png"},
		visible = false,
	})
	
	window.lastChar = nil
	window.showCount = 0

	-- Create context menu for the window
	window.tooltip = alt_CreateFrame({
		type = "Frame", name = "sm_tooltip", parent = window,
		layer = 10,
		r = 0, g = 0, b = 0, a = 1,
		hide = true,
		alignTarget = UIParent, alignTo = "TOPLEFT", align = "TOPLEFT", x = 40, y = 120
	})
	window.tooltipBG = alt_CreateFrame({
		type = "Frame", name = "sm_tooltip_bg", parent = window,
		layer = 9,
		r = 0, g = 0, b = 0, a = 1,
		border = 2, borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		hide = true,
		padTarget = window.tooltip, pad = 14 
	})
	window.tooltip.title = alt_CreateFrame({
		type = "Text", name = "windowTooltipTitle", parent = window.tooltip,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Wife Aggro", fontSize = 17, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 0
	})
	
	-- Create tabs to show on the context menu
	window.tooltip.tab1 = alt_CreateFrame({
		type = "Frame", name = "WindowTab1", parent = window.tooltip,
		layer = 10,
		r = 0, g = 0, b = 0, a = 1,
		hide = false,
		alignTarget = window.tooltip,  alignTo = "TOPLEFT", align = "TOPLEFT", x = 10, y = 40,
		borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		borderTop = 2, borderBottom = 0, borderLeft = 2, borderRight = 2,
		width = 80, height = 24,
	})
	window.tooltip.tab2 = alt_CreateFrame({
		type = "Frame", name = "WindowTab2", parent = window.tooltip,
		layer = 10,
		r = 0, g = 0, b = 0, a = 1,
		hide = false,
		alignTarget = window.tooltip.tab1, alignTo = "TOPRIGHT", align = "TOPLEFT", x = 10, y = 0,
		borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		borderTop = 2, borderBottom = 0, borderLeft = 2, borderRight = 2,
		width = 80, height = 24,
	})
	window.tooltip.tab3 = alt_CreateFrame({
		type = "Frame", name = "WindowTab3", parent = window.tooltip,
		layer = 10,
		r = 0, g = 0, b = 0, a = 1,
		hide = false,
		alignTarget = window.tooltip.tab2, alignTo = "TOPRIGHT", align = "TOPLEFT", x = 10, y = 0,
		borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		borderTop = 2, borderBottom = 0, borderLeft = 2, borderRight = 2,
		width = 80, height = 24,
	})

	window.tooltip.tab1.body = alt_CreateFrame({
		type = "Frame", name = "WindowTab1Body", parent = window.tooltip.tab1,
		layer = 11,
		r = 0, g = 0, b = 0, a = 1,
		hide = false,
		alignTarget = window.tooltip.tab1, alignTo = "BOTTOMLEFT", align = "TOPLEFT", x = -10, y = -2,
		borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		border = 2,
		width = 310, height = 220,
	})
	window.tooltip.tab2.body = alt_CreateFrame({
		type = "Frame", name = "WindowTab1Body", parent = window.tooltip.tab2,
		layer = 11,
		r = 0, g = 0, b = 0, a = 1,
		hide = true,
		alignTarget = window.tooltip.tab1.body, alignTo = "TOPLEFT", align = "TOPLEFT", x = 0, y = 0,
		borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		border = 2,
		width = window.tooltip.tab1.body:GetWidth(), height = window.tooltip.tab1.body:GetHeight(),
	})
	window.tooltip.tab3.body = alt_CreateFrame({
		type = "Frame", name = "WindowTab1Body", parent = window.tooltip.tab3,
		layer = 11,
		r = 0, g = 0, b = 0, a = 1,
		hide = true,
		alignTarget = window.tooltip.tab1.body, alignTo = "TOPLEFT", align = "TOPLEFT", x = 0, y = 0,
		borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		border = 2,
		width = window.tooltip.tab1.body:GetWidth(), height = window.tooltip.tab1.body:GetHeight(),
	})

	window.tooltip.tab1.text = alt_CreateFrame({
		type = "Text", name = "WindowTab1Text", parent = window.tooltip.tab1,
		layer = 13,
		r = 0.7, g = 0.7, b = 0.4, a = 0.9,
		fontR = 0, fontG = 0, fontB = 0, fontA = 0.9, 
		hide = false,
		alignTarget = window.tooltip.tab1, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 4,
		text = "Stopwatch", 
	})
	window.tooltip.tab2.text = alt_CreateFrame({
		type = "Text", name = "WindowTab2Text", parent = window.tooltip.tab2,
		layer = 13,
		r = 0, g = 0, b = 0, a = 0,
		fontR = .7, fontG = .7, fontB = .7, fontA = 1, 
		hide = false,
		alignTarget = window.tooltip.tab2, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 4,
		text = "Countdown", 
	})
	window.tooltip.tab3.text = alt_CreateFrame({
		type = "Text", name = "WindowTab3Text", parent = window.tooltip.tab3,
		layer = 13,
		r = 0, g = 0, b = 0, a = 0,
		fontR = .7, fontG = .7, fontB = .7, fontA = 1, 
		hide = false,
		alignTarget = window.tooltip.tab3, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 4,
		text = "Settings", 
	})

	-- Create objects to show on the stopwatch tab
	window.tooltip.tab1.body.startButton = alt_CreateFrame({
		type = "RiftButton", name = "windowTooltipStartButton", parent = window.tooltip.tab1.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Start Stopwatch", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.tab1.body, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 6
	})
	window.tooltip.tab1.body.resetButton = alt_CreateFrame({
		type = "RiftButton", name = "windowTooltipStopButton", parent = window.tooltip.tab1.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Reset Stopwatch", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.tab1.body.startButton, alignTo = "BOTTOMCENTER", align = "TOPCENTER", x = 0, y = 0
	})
	
	-- Create objects to show on the countdown tab
	window.tooltip.timerLength = RiftFrames:CreateFrame({
		type = "Text", name = "windowTooltipTimerLength", parent = window.tooltip.tab2.body,
		align = {alignTo = "TOPCENTER", target = window.tooltip.tab2.body, align = "TOPCENTER", x = 0, y = 5, },
		text = "00:00:00", fontSize = 13, wordwrap = false,
		width = 90, 
		background = {0,0,0,0},
	})
	
	window.tooltip.timerLengthHours = RiftFrames:CreateFrame({
		type = "RiftSlider", name = "windowTooltipTimerHours", parent = window.tooltip.tab2.body,
		align = {alignTo = "TOPCENTER", target = window.tooltip.timerLength, align = "BOTTOMCENTER", x = 0, y = -15},
		range = {0,23}, position = 0,
		width = window.tooltip.tab1.body.resetButton:GetWidth(),
	})

	window.tooltip.timerLengthMinutes = UI.CreateFrame("RiftSlider","windowTooltipTimerMinutes",window.tooltip.tab2.body)
	window.tooltip.timerLengthMinutes:SetPoint("TOPCENTER", window.tooltip.timerLengthHours, "BOTTOMCENTER", 0, 0)
	window.tooltip.timerLengthMinutes:SetRange(0,59)
	window.tooltip.timerLengthMinutes:SetWidth(window.tooltip.tab1.body.resetButton:GetWidth())
	window.tooltip.timerLengthMinutes:SetPosition(0)

	--window.tooltip.timerLengthSeconds = RiftFrames.CreateFrame({
	--	type = "RiftSlider", name = "windowTooltipTimerSeconds", parent = window.tooltip.tab2.body,
	--	align = {alignTo = "TOPCENTER", target = window.tooltip.timerLengthMinutes, align = "BOTTOMCENTER", x = 0, y = 0},
	--	range = {0,59}, position = 0,
	--	width = window.tooltip.tab1.body.resetButton:GetWidth(),
	--})
	window.tooltip.timerLengthSeconds = UI.CreateFrame("RiftSlider","windowTooltipTimerSeconds",window.tooltip.tab2.body)
	window.tooltip.timerLengthSeconds:SetPoint("TOPCENTER", window.tooltip.timerLengthMinutes, "BOTTOMCENTER", 0, 0)
	window.tooltip.timerLengthSeconds:SetRange(0,59)
	window.tooltip.timerLengthSeconds:SetWidth(window.tooltip.tab1.body.resetButton:GetWidth())
	window.tooltip.timerLengthSeconds:SetPosition(0)

	window.tooltip.timerLoop = alt_CreateFrame({
		type = "Frame", name = "windowTooltipTimerLoop", parent = window.tooltip.tab2.body,
		layer = 10,
		r = 1, g = 0, b = 0, a = 0,
		hide = false,
		alignTarget = window.tooltip.timerLengthSeconds, alignTo = "BOTTOMLEFT", align = "TOPLEFT", x = 0, y = 0
	})
	window.tooltip.timerLoopTimer = alt_CreateFrame({
		type = "RiftCheckbox", name = "windowTooltipTimerLoopTimer", parent = window.tooltip.timerLoop,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		hide = false,
		alignTarget = window.tooltip.timerLoop, alignTo = "TOPLEFT", align = "TOPLEFT", x = 0, y = 8
	})
	window.tooltip.timerLoopTimerText = alt_CreateFrame({
		type = "Text", name = "windowTooltipTimerLoopTimerText", parent = window.tooltip.timerLoopTimer,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Restart countdown on finish", 
		hide = false,
		alignTarget = window.tooltip.timerLoopTimer, alignTo = "TOPRIGHT", align = "TOPLEFT", x = 0, y = -1
	})
	window.tooltip.setCountdown = alt_CreateFrame({
		type = "RiftButton", name = "windowTooltipSetCountdown", parent = window.tooltip.timerLoop,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Start", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.timerLengthSeconds, alignTo = "BOTTOMCENTER", align = "TOPCENTER", x = 0, y = 26
	})
	window.tooltip.resetCountdown = alt_CreateFrame({
		type = "RiftButton", name = "windowTooltipResetCountdown", parent = window.tooltip.timerLoop,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Reset", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.setCountdown, alignTo = "BOTTOMCENTER", align = "TOPCENTER", x = 0, y = -3
	})
	window.tooltip.clearWarning = alt_CreateFrame({
		type = "RiftButton", name = "windowTooltipClearWarning", parent = window.tooltip.tab2.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Clear Warning", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.resetCountdown, alignTo = "BOTTOMCENTER", align = "TOPCENTER", x = 0, y = -3
	})

	-- Create objects for the settings tab
	window.tooltip.settingsText = alt_CreateFrame({
		type = "Text", name = "windowTooltipSettingsText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Warning message", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.tab3.body, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 8
	})
	window.tooltip.warningMessage = UI.CreateFrame("RiftTextfield","windowTooltipWarningMessage",window.tooltip.tab3.body)
	window.tooltip.warningMessage:SetPoint("TOPCENTER", window.tooltip.settingsText, "BOTTOMCENTER",0,0)
	window.tooltip.warningMessage:SetText(wife_timerSettings.warningMessage)
	window.tooltip.warningMessage:SetBackgroundColor(.12,.12,.12,1)

	window.tooltip.alphaText = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Red screen warning opacity", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.warningMessage, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 30
	})
	window.tooltip.warningAlpha = UI.CreateFrame("RiftSlider","windowTooltipWarningAlpha",window.tooltip.tab3.body)
	window.tooltip.warningAlpha:SetPoint("TOPCENTER", window.tooltip.alphaText, "BOTTOMCENTER", 0, 6)
	window.tooltip.warningAlpha:SetWidth(180)
	window.tooltip.warningAlpha:SetRange(0,50)
	window.tooltip.warningAlpha:SetPosition(math.floor(wife_timerSettings.redWarningAlpha*100))
	window.tooltip.alphaTextLow = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "0%", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.warningAlpha, alignTo = "TOPLEFT", align = "TOPRIGHT", x = -10, y = -5
	})
	window.tooltip.alphaTextHigh = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "50%", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.warningAlpha, alignTo = "TOPRIGHT", align = "TOPLEFT", x = 10, y = -5
	})
		
	window.tooltip.secondsToWarnText = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Seconds before showing warning", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.warningAlpha, alignTo = "TOPCENTER", align = "TOPCENTER", x = 0, y = 30
	})
	window.tooltip.secondsToWarn = UI.CreateFrame("RiftSlider","windowTooltipWarningLength",window.tooltip.tab3.body)
	window.tooltip.secondsToWarn:SetPoint("TOPCENTER", window.tooltip.secondsToWarnText, "BOTTOMCENTER", 0, 6)
	window.tooltip.secondsToWarn:SetWidth(180)
	window.tooltip.secondsToWarn:SetRange(0,120)
	window.tooltip.secondsToWarn:SetPosition(wife_timerSettings.warnTimeInSeconds)
	window.tooltip.secondsToWarnLow = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "0", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.secondsToWarn, alignTo = "TOPLEFT", align = "TOPRIGHT", x = -10, y = -5
	})
	window.tooltip.secondsToWarnHigh = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "120", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.secondsToWarn, alignTo = "TOPRIGHT", align = "TOPLEFT", x = 10, y = -5
	})
	window.tooltip.secondsToWarnAmount = alt_CreateFrame({
		type = "Text", name = "windowTooltipAlphaText", parent = window.tooltip.tab3.body,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "000", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip.secondsToWarn, alignTo = "BOTTOMCENTER", align = "TOPCENTER", x = 0, y = -14
	})
	window.tooltip.secondsToWarnAmount:SetText(string.format("%d",window.tooltip.secondsToWarn:GetPosition()))
	
	-- Create a close button for the context menu
	window.tooltip.closeButton = alt_CreateFrame({
		type = "RiftButton", name = "windowTooltipCloseButton", parent = window.tooltip,
		layer = 10,
		r = 0, g = 0, b = 0, a = 0,
		text = "Close", fontSize = 13, wordwrap = false,
		hide = false,
		alignTarget = window.tooltip, alignTo = "BOTTOMCENTER", align = "BOTTOMCENTER", x = 0, y = 0
	})

	-- Create an alert window to alarm when timer times out
	window.alerter = alt_CreateFrame({
		type = "Frame", name = "windowTooltipClearWarning", parent = window,
		layer = 25,
		r = 1, g = 0, b = 0, a = 1,
		hide = true,
		alignTarget = UIParent, alignTo = "CENTER", align = "CENTER", x = 0, y = 0
	})
	window.alerter:SetAlpha(wife_timerSettings.redWarningAlpha)
	window.alerter:SetWidth(UIParent:GetWidth())
	window.alerter:SetHeight(UIParent:GetHeight())
	window.alerter.warning = alt_CreateFrame({
		type = "Text", name = "alerterWarning", parent = window,
		layer = 26,
		r = 0, g = 0, b = 0, a = .7,
		text = wife_timerSettings.warningMessage, fontSize = 36, wordwrap = false,
		hide = true,
		alignTarget = window.alerter, alignTo = "CENTER", align = "CENTER", x = 0, y = 0
	})

	-- General Window Functions
	function window:fSetTime(inHr, inMn, inSc)
		tm = {}
		if inHr and inMn and inSc then
			tm.hrs = inHr
			tm.mins = inMn
			tm.secs = inSc
		else
			tm.hrs = window.tooltip.timerLengthHours:GetPosition()
			tm.mins = window.tooltip.timerLengthMinutes:GetPosition()
			tm.secs = window.tooltip.timerLengthSeconds:GetPosition()
		end
		val = string.format("%.2d:%.2d:%.2d",tm.hrs,tm.mins,tm.secs)
		return val
	end
	function window:fGetTime()
		tm = window.tooltip.timerLengthHours:GetPosition()*3600
		tm = tm + window.tooltip.timerLengthMinutes:GetPosition()*60
		tm = tm + window.tooltip.timerLengthSeconds:GetPosition()
		return tm
	end
	function window:fSetMode(mode)
		if mode == "stopwatch" then
			window.tooltip.setCountdown:SetEnabled(false)
			window.tooltip.resetCountdown:SetEnabled(false)
			window.tooltip.timerLengthHours:SetEnabled(false)
			window.tooltip.timerLengthMinutes:SetEnabled(false)
			window.tooltip.timerLengthSeconds:SetEnabled(false)
			window.tooltip.timerLoopTimer:SetEnabled(false)
			window.tooltip.tab1.body.resetButton:SetEnabled(false)
			window.title.icon.clock:SetVisible(false)
			window.title.icon.circle:SetVisible(false)
			window.title.icon.up:SetVisible(true)
			window.title.icon.down:SetVisible(false)
		elseif mode == "countdown" then
			window.tooltip.tab1.body.startButton:SetEnabled(false)
			window.tooltip.tab1.body.resetButton:SetEnabled(false)
			window.tooltip.timerLengthHours:SetEnabled(false)
			window.tooltip.timerLengthMinutes:SetEnabled(false)
			window.tooltip.timerLengthSeconds:SetEnabled(false)
			window.tooltip.timerLoopTimer:SetEnabled(false)
			window.title.icon.clock:SetVisible(false)
			window.title.icon.circle:SetVisible(window.tooltip.timerLoopTimer:GetChecked())
			window.title.icon.up:SetVisible(false)
			window.title.icon.down:SetVisible(not window.tooltip.timerLoopTimer:GetChecked())
		else
			window.tooltip.tab1.body.startButton:SetEnabled(true)
			window.tooltip.tab1.body.resetButton:SetEnabled(true)
			window.tooltip.setCountdown:SetEnabled(true)
			window.tooltip.resetCountdown:SetEnabled(true)
			window.tooltip.timerLengthHours:SetEnabled(true)
			window.tooltip.timerLengthMinutes:SetEnabled(true)
			window.tooltip.timerLengthSeconds:SetEnabled(true)
			window.tooltip.timerLoopTimer:SetEnabled(true)
			window.title.icon.clock:SetVisible(true)
			window.title.icon.circle:SetVisible(false)
			window.title.icon.up:SetVisible(false)
			window.title.icon.down:SetVisible(false)
		end
	end
	function window:fShowTooltip(text)
		--window.showCount = window.showCount + 1
		self.tooltip:SetWidth(310);
		self.tooltip:SetHeight(320)

		self.tooltipBG:SetVisible(true)
		self.tooltip:SetVisible(true)
	end
	function window:fHideTooltip()
		--window.showCount = window.showCount - 1
		
		--if window.showCount < 1 then
			self.tooltip:SetVisible(false)
			self.tooltipBG:SetVisible(false)
		--end
	end
	function window:fPositionTooltip(posX,posY)
		self.tooltip:SetPoint("TOPLEFT",UIParent,"TOPLEFT",posX+20,posY+20)
	end
	function window.fShow()
		window:SetVisible(true)
		wife_windowSettings.state = 'show'
	end
	function window.fHide()
		window:SetVisible(false)
		wife_windowSettings.state = 'hide'
	end
	function window:fChangeTab(tabNumber)
		tabCount = 3
		if tabNumber == 1 then
			window.tooltip.tab1.text:SetBackgroundColor(0.7,0.7,0.4,0.9)
			window.tooltip.tab1.text:SetFontColor(0,0,0,0)
			window.tooltip.tab1.body:SetVisible(true)
			window.tooltip.tab2.text:SetBackgroundColor(0,0,0,0.9)
			window.tooltip.tab2.text:SetFontColor(1,1,1,1)
			window.tooltip.tab2.body:SetVisible(false)
			window.tooltip.tab3.text:SetBackgroundColor(0,0,0,0.9)
			window.tooltip.tab3.text:SetFontColor(1,1,1,1)
			window.tooltip.tab3.body:SetVisible(false)
		elseif tabNumber == 2 then
			window.tooltip.tab1.text:SetBackgroundColor(0,0,0,0.9)
			window.tooltip.tab1.text:SetFontColor(1,1,1,1)
			window.tooltip.tab1.body:SetVisible(false)
			window.tooltip.tab2.text:SetBackgroundColor(0.7,0.7,0.4,0.9)
			window.tooltip.tab2.text:SetFontColor(0,0,0,0)
			window.tooltip.tab2.body:SetVisible(true)
			window.tooltip.tab3.text:SetBackgroundColor(0,0,0,0.9)
			window.tooltip.tab3.text:SetFontColor(1,1,1,1)
			window.tooltip.tab3.body:SetVisible(false)
		elseif tabNumber == 3 then
			window.tooltip.tab1.text:SetBackgroundColor(0,0,0,0.9)
			window.tooltip.tab1.text:SetFontColor(1,1,1,1)
			window.tooltip.tab1.body:SetVisible(false)
			window.tooltip.tab2.text:SetBackgroundColor(0,0,0,0.9)
			window.tooltip.tab2.text:SetFontColor(1,1,1,1)
			window.tooltip.tab2.body:SetVisible(false)
			window.tooltip.tab3.text:SetBackgroundColor(0.7,0.7,0.4,0.9)
			window.tooltip.tab3.text:SetFontColor(0,0,0,0)
			window.tooltip.tab3.body:SetVisible(true)
		end
	end
	function window:fTriggerCountdown()
		if not isStopWatchRunning then
			if not isTimerRunning then
				-- The timer isn't running, so let's start it up
				if TimerTimeRemaining == nil then
					-- We didn't resume a timer, so start from the beginning
					tTime=3600*window.tooltip.timerLengthHours:GetPosition()+60*window.tooltip.timerLengthMinutes:GetPosition()+window.tooltip.timerLengthSeconds:GetPosition()
				else
					-- We are resuming a countdown, so pick up where we left off
					times = explode(":",window.timerFrame:GetText())
					tTime=3600*times[1]+60*times[2]+times[3]
				end
				if tTime > 0 then
					TimerStartTime = Inspect.Time.Real()+tTime
					isTimerRunning = true
					window:fSetMode("countdown")
					window.tooltip.resetCountdown:SetEnabled(false)
					window.tooltip.setCountdown:SetText("Stop")
				end
				window:fHideTooltip()
			else
				-- The timer is running, so let's stop it now
				isTimerRunning = false
				window:fSetMode("")
				window.tooltip.resetCountdown:SetEnabled(true)
				TimerTimeRemaining = window.timerFrame:GetText()
				window.tooltip.setCountdown:SetText("Start")
			end
		end
	end
	function window:fResetTimer()
		if not isTimerRunning then 
			if isStopWatchRunning then
			else
				window.timerFrame:SetText(window:fSetTime())
			end
		end
	end
	function window:fResetStopwatch()
		if not isTimerRunning then 
			if isStopWatchRunning then
			else
				window.timerFrame:SetText("00:00:00")
			end
		end
	end
	
	-- Window Title Event Functions
	--     In general, these are involved in moving the timer and opening the context menu
	function window.title.Event:LeftDown()
		self.MouseDown = true
		local mouse = Inspect.Mouse()
		window.startX = window:GetLeft()
		window.startY = window:GetTop()
		window.mouseStartX = mouse.x
		window.mouseStartY = mouse.y
	end
	function window.title.Event:MouseMove()
		if self.MouseDown then
			local mouse = Inspect.Mouse()
			wife_windowSettings.posX = mouse.x - window.mouseStartX + window.startX
			wife_windowSettings.posY = mouse.y - window.mouseStartY + window.startY
			window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", wife_windowSettings.posX, wife_windowSettings.posY)
		end
	end
	function window.title.Event:LeftUp()
		if self.MouseDown then
			self.MouseDown = false
		end
	end
	function window.title.Event:MouseIn()
		window.title:SetBackgroundColor(1,1,1,.3)
	end
	function window.title.Event:MouseOut()
		window.title:SetBackgroundColor(1,1,1,.1)
	end
	function window.title.Event:RightClick()
		local mouse = Inspect.Mouse()
		window:fPositionTooltip(mouse.x-10,mouse.y-10)
		window:fShowTooltip()
	end

	-- Stopwatch event handlers
	function window.tooltip.tab1.body.startButton.Event:LeftClick()
		if not isTimerRunning then 
			if isStopWatchRunning then
				window:fSetMode("")
				window.tooltip.tab1.body.startButton:SetText("Start Stopwatch")
				isStopWatchRunning = false
			else
				window:fSetMode("stopwatch")
				window.tooltip.tab1.body.startButton:SetText("Stop Stopwatch")
				isStopWatchRunning = true
				if TimerStartTime == nil then 
					TimerStartTime = Inspect.Time.Real() 
				else
					timeStuff = explode(":",window.timerFrame:GetText())
					TimerStartTime = Inspect.Time.Real() - timeStuff[1]*3600 - timeStuff[2]*60 - timeStuff[3]
				end
				window:fHideTooltip()
			end
		end
	end
	function window.tooltip.tab1.body.resetButton.Event:LeftClick()
		window:fResetStopwatch()
	end

	-- Countdown event handlers
	function window.tooltip.timerLengthHours.Event:SliderChange()
		window.tooltip.timerLength:SetText(window:fSetTime())
		window.timerFrame:SetText(window:fSetTime())
	end
	function window.tooltip.timerLengthMinutes.Event:SliderChange()
		window.tooltip.timerLength:SetText(window:fSetTime())
		window.timerFrame:SetText(window:fSetTime())
	end
	function window.tooltip.timerLengthSeconds.Event:SliderChange()
		window.tooltip.timerLength:SetText(window:fSetTime())
		window.timerFrame:SetText(window:fSetTime())
	end
	function window.tooltip.closeButton.Event:LeftClick()
		window:fHideTooltip()
	end
	function window.tooltip.setCountdown.Event:LeftClick()
		window:fTriggerCountdown()
	end
	function window.tooltip.resetCountdown.Event:LeftClick()
		window:fResetTimer()
	end
	function window.tooltip.clearWarning.Event:LeftClick()
		window.alerter:SetVisible(false)
		window.alerter.warning:SetVisible(false)
	end
	function window.tooltip.timerLoopTimer.Event:CheckboxChange()
	end
	
	-- Settings event handlers
	function window.tooltip.warningMessage.Event:TextfieldChange()
		wife_timerSettings.warningMessage = window.tooltip.warningMessage:GetText()
		window.alerter.warning:SetText(wife_timerSettings.warningMessage)
		window.alerter.warning:ResizeToText()
	end
	function window.tooltip.warningAlpha.Event:SliderChange()
		wife_timerSettings.redWarningAlpha = window.tooltip.warningAlpha:GetPosition()/100
		window.alerter:SetAlpha(wife_timerSettings.redWarningAlpha)
	end
	function window.tooltip.secondsToWarn.Event:SliderChange()
		wife_timerSettings.warnTimeInSeconds = window.tooltip.secondsToWarn:GetPosition()
		window.tooltip.secondsToWarnAmount:SetText(string.format("%d",window.tooltip.secondsToWarn:GetPosition()))
	end

	-- Tab switching event handlers
	function window.tooltip.tab1.text.Event:MouseIn()
	end
	function window.tooltip.tab1.text.Event:MouseOut()
	end
	function window.tooltip.tab1.text.Event:LeftClick()
		window:fChangeTab(1)
	end

	function window.tooltip.tab2.text.Event:MouseIn()
	end
	function window.tooltip.tab2.text.Event:MouseOut()
	end
	function window.tooltip.tab2.text.Event:LeftClick()
		window:fChangeTab(2)
	end

	function window.tooltip.tab3.text.Event:MouseIn()
	end
	function window.tooltip.tab3.text.Event:MouseOut()
	end
	function window.tooltip.tab3.text.Event:LeftClick()
		window:fChangeTab(3)
	end

	window:SetVisible(wife_windowSettings.state == 'show')
end


-- ------------------------------------------------------------------
-- Initialization Function
-- ------------------------------------------------------------------
local function loadSettings()
	myName = Inspect.Addon.Current()
	print ("Welcome to " .. myName .. "!")
	if wife_windowSettings.state then
		if wife_windowSettings.state == 'hide' then
			print (myName .. " is currently hidden.  Use /alt show to display the window")
		end
	else
		wife_windowSettings.state = 'show'
	end
	altCreateWindow()
end
local function altPlayerAvailable(params)
	for k,v in pairs(params) do
		if v=="player" then
			altSavePlayerData()
			window.fUpdateList()
		end
	end
end


-- ------------------------------------------------------------------
-- Slash functions
-- ------------------------------------------------------------------
local function setTimer(params)
	if not isTimerRunning and not isStopWatchRunning then
		if params then
			tParams = explode(":",params)
			if #tParams > 1 and #tParams < 4 then
				if #tParams == 3 then
					tParams[1] = math.floor(tParams[1])
					tParams[2] = math.floor(tParams[2])
					tParams[3] = math.floor(tParams[3])
				else
					tParams[3] = math.floor(tParams[2])
					tParams[2] = math.floor(tParams[1])
					tParams[1] = 0
				end
				if tParams[1] >= 0 and tParams[1] < 24 and tParams[2] >= 0 and tParams[2] < 60 and tParams[3] >= 0 and tParams[3] < 60 then
					window.tooltip.timerLengthHours:SetPosition(tParams[1])
					window.tooltip.timerLengthMinutes:SetPosition(tParams[2])
					window.tooltip.timerLengthSeconds:SetPosition(tParams[3])
					window.tooltip.timerLength:SetText(window.fSetTime(tParams[1],tParams[2],tParams[3]))
					window.timerFrame:SetText(window.fSetTime(tParams[1],tParams[2],tParams[3]))
				end
			end
		end
	end
end
local function startTimer()
	if not isTimerRunning then
		window:fTriggerCountdown()
	end
end
local function stopTimer()
	if isTimerRunning then
		window:fTriggerCountdown()
	end
end
local function resetTimer()
	if not isTimerRunning and not isStopwWtchRunning then
		window:fResetTimer()
	end
end
local function autoTimer(params)
	if not isTimerRunning then
		if params then
			if string.lower(params) == "on" then
				window.tooltip.timerLoopTimer:SetChecked(true)
			elseif string.lower(params) == "off" then
				window.tooltip.timerLoopTimer:SetChecked(false)
			end
		else
			window.tooltip.timerLoopTimer:SetChecked(not window.tooltip.timerLoopTimer:GetChecked())
		end
	end
end


-- ------------------------------------------------------------------
-- Slash handler
-- ------------------------------------------------------------------
local function altSlashCommand(params)
	tParams = explode(" ",params)
	if tParams[1] == "abil" then
		getAbilities()
	elseif tParams[1] == "show" then
		window.fShow()
	elseif tParams[1] == "hide" then
		window.fHide()
	elseif tParams[1] == "init" then
		initGlobals()
	elseif tParams[1] == "set" then
		setTimer(tParams[2])
	elseif tParams[1] == "start" then
		startTimer(tParams[2])
	elseif tParams[1] == "stop" then
		stopTimer()
	elseif tParams[1] == "reset" then
		resetTimer()
	elseif tParams[1] == "auto" then
		autoTimer(tParams[2])
	else
		print ("Parameters:")
		print ("\tshow -- show the Altometer window")
		print ("\thide -- hide the Altometer window")
		print ("\tinit -- Resets SavedVariable data to defaults")
		print ("\tset [time] -- manually set the timer.  Accepts MM:SS or HH:MM:SS for time")
		print ("\tstart -- starts the countdown")
		print ("\tstop -- stops the countdown")
		print ("\treset -- resets the countdown")
		print ("\tauto [on/off] -- enables or disabled the autoreset of the countdown. Without a parameter, the option is toggled.")
	end
end
local function TimerUpdate()
	if isTimerRunning or isStopWatchRunning then
		currTime = math.abs(Inspect.Time.Real()-TimerStartTime)
		hours = math.floor(currTime/3600)
		minutes = math.floor((currTime-hours*3600)/60)
		seconds = math.floor(currTime-hours*3600-minutes*60)
		val = string.format("%.2d:%.2d:%.2d",hours,minutes,seconds)
		if isTimerRunning and (Inspect.Time.Real() > TimerStartTime) then
			-- If we get here, the timer has expired, so let's get our next action
			if window.tooltip.timerLoopTimer:GetChecked() then
				-- The user wants to restart the timer
				TimerStartTime = Inspect.Time.Real()+3600*window.tooltip.timerLengthHours:GetPosition()+60*window.tooltip.timerLengthMinutes:GetPosition()+window.tooltip.timerLengthSeconds:GetPosition()
			else
				-- The user doesn't want to restart the timer, so switch to stopwatch mode
				window:fSetMode("")
				window:fSetMode("stopwatch")
				window.tooltip.tab1.body.startButton:SetText("Stop Stopwatch")
				isTimerRunning = false
				isStopWatchRunning = true
			end
		end
		if isTimerRunning and hours == 0 and currTime <= wife_timerSettings.warnTimeInSeconds then
			-- We are in countdown mode, in the middle of the warning period
			window.alerter:SetVisible(true)
			window.alerter.warning:SetVisible(true)
		else
			-- We are in countdown mode, but not in the middle of the warnign period
			window.alerter:SetVisible(false)
			window.alerter.warning:SetVisible(false)
		end
		if isTimerRunning then
			-- Let's update the background bar
			window.timerFrameBackground:SetWidth(currTime / window:fGetTime() * window.timerFrame:GetWidth())
		end
		-- Update the display according to the new time to display
		window.timerFrame:SetText(val)
	end
end


-- ------------------------------------------------------------------
-- Table Insertions
-- ------------------------------------------------------------------
table.insert (Command.Slash.Register("wife"), {altSlashCommand, "WifeAggro", "SlashCommand"})
table.insert (Event.Addon.Startup.End, {loadSettings, "WifeAggro", "loadSettings"})
table.insert (Event.System.Update.Begin, {TimerUpdate, "WifeAggro", "TimerUpdate"})
table.insert (Event.Addon.SavedVariables.Load.End, {AddonsLoadedDone, "WifeAggro", "LoadAddonsDone"})

